import os
import numpy as np
import re
from . import utils, npz_converter
from tqdm import tqdm

"""
Saves flow cytometry data in compressed npz file.
Contains ndarrays with the following keys.

key: channel_names
String list of channel names.
Used as attributes in FCS_Data class.

key: sample_rate
int;
Sampling rate in hertz.

key: time_stamp (optional)
1D array [number of segments];
Time stamp of each segment in sampling points.
Multiply by the sampling interval to get the actual time.

key: [each name in channel_names]
2D array [number of segments, segment size];
Voltage data of each channel in millivolts.
"""


class FCS_Data:


    KEY_CHANNEL_NAMES = 'channel_names'
    KEY_SAMPLE_RATE = 'sample_rate'
    KEY_TIME_STAMP = 'time_stamp'

    KEY_FSC = 'FSC'
    KEY_SSC = 'SSC'
    KEY_SPC_MOD = 'SPC-mod'
    KEY_SSC_PATTERN = 'SSC_pattern'

    SUFFIX_A = '_a'
    SUFFIX_H = '_h'
    SUFFIX_W = '_w'
    SUFFIX_PSNR = '_psnr'


    def __init__(self, channel_names, sample_rate, **kwargs):
        self.channel_names = list(channel_names)
        self.sample_rate = int(sample_rate)
        for key in kwargs:
            utils.set_attr(self, key, kwargs.get(key))


    @classmethod
    def from_file(cls, *files):
        data_dict = npz_converter.get_combined_data(*files)
        data = cls(**data_dict)
        data.create_subattributes()
        return data


    @classmethod
    def from_data_array(cls, channel_names, sample_rate, data_array, time_stamp=None):
        data = utils.array_to_dict(channel_names, data_array, axis=1)
        return cls(channel_names, sample_rate, time_stamp=time_stamp, **data)


    def create_subattributes(self):
        for key in self.channel_names:
            wf = utils.get_attr(self, key)
            if key in {self.KEY_SPC_MOD,}:
                bg_wf = wf[:, :100]
                N = np.median(np.std(bg_wf, axis=1))
                ABS = np.abs(wf)
                H = np.max(ABS, axis=1)
                sig = np.copy(ABS)
                sig[sig < N * 3] = 0
                A = np.sum(sig, axis=1)
                psnr = (H / N) ** 2
                utils.set_attr(self, key + self.SUFFIX_H, H)
                utils.set_attr(self, key + self.SUFFIX_A, A)
                utils.set_attr(self, key + self.SUFFIX_PSNR, psnr)
            else:
                if key in {self.KEY_SSC_PATTERN,}:
                    bg_wf = wf[:, :100]
                    bg = np.median(bg_wf)
                else:
                    bg = np.reshape(np.median(wf, axis=1), (-1, 1))
                H = np.max(wf - bg, axis=1)
                A = np.sum(wf - bg, axis=1)
                W = self._get_width(wf, bg)
                utils.set_attr(self, key + self.SUFFIX_H, H)
                utils.set_attr(self, key + self.SUFFIX_A, A)
                utils.set_attr(self, key + self.SUFFIX_W, W)


    def save(self, fn):
        data_dict = dict()

        for key in [self.KEY_CHANNEL_NAMES, self.KEY_SAMPLE_RATE] + self.channel_names:
            data_dict.update({key: utils.get_attr(self, key)})

        ts = utils.get_attr(self, self.KEY_TIME_STAMP, None)
        if ts:
            data_dict.update({self.KEY_TIME_STAMP: ts})

        np.savez_compressed(fn, **data_dict)


    def get_waveform(self, ch, n):
        return utils.get_attr(self, ch)[n]

    def get_attr(self, attr):
        return utils.get_attr(self, attr)


    def _get_width(self, wf, bg):
        length = wf.shape[1] // 10
        std = np.median(np.std(bg, axis=1))

        thresh = std * 6
        width = []
        for s in wf:
            w, = np.where(s > thresh)
            if len(w) > 0:
                start = w[0]
                end = w[-1]
            else:
                start = 0
                end = 0
            width.append(end - start)

        return np.asarray(width)


    def get_fsc_width(self):
        length = self.fsc.shape[1] // 10
        bg = self.fsc[:, :length]
        fsc_std = np.median(np.std(bg, axis=1))
        
        thresh = fsc_std * 6
        width = []
        for s in self.fsc:
            w, = np.where(s > thresh)
            if len(w) > 0:
                start = w[0]
                end = w[-1]
            else:
                start = 0
                end = 0
            width.append(end - start)
        self.fsc_width = np.asarray(width)


    def get_singlets_spc(self):
        mean_start = np.reshape(np.median(np.mean(self.spc_mod[:, :50] , axis=1)), (-1, 1))
        mean_end   = np.reshape(np.median(np.mean(self.spc_mod[:, -50:], axis=1)), (-1, 1))
        bg_start = np.median(np.std(self.spc_mod[:, :50]  - mean_start, axis=1))
        bg_end   = np.median(np.std(self.spc_mod[:, -50:] - mean_end, axis=1))
        start_thresh = 50
        end_thresh = 870
        med_start = np.median(mean_start)
        med_end = np.median(mean_end)

        singlets_spc = []
        for x in tqdm(self.spc_mod):
            w_s, = np.where(np.abs(x - np.median(mean_start)) > bg_start * 6)
            w_e, = np.where(np.abs(x - np.median(mean_end)) > bg_end * 6)
            if len(w_s) == 0 or len(w_e) == 0:
                singlets_spc.append(False)
            else:
                start = w_s[0]
                end = w_e[-1]

                if start < start_thresh or end > end_thresh:
                    singlets_spc.append(False)
                else:
                    singlets_spc.append(True)

        self.singlets_spc = np.asarray(singlets_spc)