import sys
import numpy as np
import time

# import spectrum driver functions
from .pyspcm import *
from . import converter as cv


class M2i:
    __instance = None

    def __init__(self):
        # open card
        hCard = spcm_hOpen(create_string_buffer(b'/dev/spcm0'))
        if hCard == None:
            sys.stdout.write("no card found...\n")
            return
            # exit()
        self.hCard = hCard
        self._check_card()

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance


    def _check_card(self):
        # read type, function and sn and check for A/D card
        lCardType = int32(0)
        spcm_dwGetParam_i32(self.hCard, SPC_PCITYP, byref(lCardType))
        lSerialNumber = int32(0)
        spcm_dwGetParam_i32(self.hCard, SPC_PCISERIALNO, byref(lSerialNumber))
        lFncType = int32(0)
        spcm_dwGetParam_i32(self.hCard, SPC_FNCTYPE, byref(lFncType))
        lInstMemsize = int64(0)
        spcm_dwGetParam_i64(self.hCard, SPC_PCIMEMSIZE, byref(lInstMemsize))
        self.max_mem_size = lInstMemsize.value

        sCardName = cv.szTypeToName(lCardType.value)
        if lFncType.value == SPCM_TYPE_AI:
            sys.stdout.write("Found: {0} sn {1:05d}\n".format(sCardName, lSerialNumber.value))
            sys.stdout.write("Memory on card: {0:.1f} M Samples\n".format(self.max_mem_size / 1024 / 1024 / 2))
        else:
            sys.stdout.write("Card: {0} sn {1:05d} not supported by example\n".format(sCardName, lSerialNumber.value))
            # exit()


    def set_channels(self, n_channels=4, channels=None):
        self.n_channels = n_channels
        self.input_ranges = np.zeros(n_channels)
        spcm_dwSetParam_i32(self.hCard, SPC_CHENABLE, cv.get_hex_channels(n_channels, channels))


    def set_input(self, channel, termination='1M', v_range=1000):
        # Check if channel is valid
        if channel in {0, 1, 2, 3, 4, 5, 6, 7}:
            n = channel * 100
        else:
            sys.stdout.write("Specify valid channel.\n")
            return

        # Set termination
        spcm_dwSetParam_i32(self.hCard, SPC_50OHM0 + n, cv.get_bin_termination(termination))

        # Set input range
        spcm_dwSetParam_i32(self.hCard, SPC_AMP0 + n, v_range)
        self.input_ranges[channel] = v_range


    def set_aquisition(self, segment_size, pretrigger, n_segments, sample_rate=250):

        lSamplerate = int64(0)

        # Aquisition mode
        spcm_dwSetParam_i32(self.hCard, SPC_CARDMODE, SPC_REC_STD_MULTI)
        spcm_dwSetParam_i32(self.hCard, SPC_TIMEOUT, 5000)  # timeout 5 s

        # Set sample length
        self.segment_size = segment_size
        self.n_segments = n_segments
        self.mem_size = segment_size * n_segments

        if self.mem_size * self.n_channels * 2 > self.max_mem_size:
            sys.stdout.write("Error: Exceeds available memory size.\n")
            return

        spcm_dwSetParam_i32(self.hCard, SPC_MEMSIZE, self.mem_size)
        spcm_dwSetParam_i32(self.hCard, SPC_SEGMENTSIZE, segment_size)
        spcm_dwSetParam_i32(self.hCard, SPC_POSTTRIGGER, segment_size - pretrigger)

        # Clockmode
        spcm_dwSetParam_i32(self.hCard, SPC_CLOCKMODE, SPC_CM_INTPLL)  # clock mode internal PLL

        # Set the samplerate to division of max (250 MHz) on internal PLL, no clock output
        spcm_dwSetParam_i64(self.hCard, SPC_SAMPLERATE, MEGA(sample_rate))
        spcm_dwSetParam_i32(self.hCard, SPC_CLOCKOUT, 0)  # no clock output
        spcm_dwGetParam_i64(self.hCard, SPC_SAMPLERATE, byref(lSamplerate))
        sample_rate = lSamplerate.value
        sys.stdout.write("Set sampling rate to {0:.3} MHz.\n".format(sample_rate / 10 ** 6))
        sys.stdout.write("Sampling interval will be {0:.3} ns.\n".format(1 / sample_rate * 10 ** 9))
        return sample_rate


    def set_trigger(self, channel=0, mode='pos', mv_level=0):
        # disable software trigger
        spcm_dwSetParam_i32(self.hCard, SPC_TRIG_ORMASK, SPC_TMASK_NONE)

        n = channel
        level = cv.mvolt_to_digitized_level(mv_level, self.input_ranges[channel], bits=14)
        spcm_dwSetParam_i32(self.hCard, SPC_TRIG_CH_ORMASK0, cv.get_hex_trig_channel(channel))
        spcm_dwSetParam_i32(self.hCard, SPC_TRIG_CH0_MODE + n, cv.get_hex_trig_mode(mode))
        spcm_dwSetParam_i32(self.hCard, SPC_TRIG_CH0_LEVEL0 + n, level)


    def _set_buffer(self):
        # settings for the DMA buffer
        self.lNotifySize = int32(0); # driver should notify program after all data has been transfered

        # define the data buffer
        # we try to use continuous memory if available and big enough
        self.qwBufferSize = uint64(self.mem_size * 2 * self.n_channels);
        self.pvBuffer = c_void_p()
        qwContBufLen = uint64(0)
        spcm_dwGetContBuf_i64(self.hCard, SPCM_BUF_DATA, byref(self.pvBuffer), byref(qwContBufLen))
        # sys.stdout.write("ContBuf length: {0:d}\n".format(qwContBufLen.value))
        if qwContBufLen.value >= self.qwBufferSize.value:
            pass
            # sys.stdout.write("Using continuous buffer\n")
        else:
            self.pvBuffer = create_string_buffer(self.qwBufferSize.value)
            # sys.stdout.write("Using buffer allocated by user program\n")

        spcm_dwDefTransfer_i64(self.hCard, SPCM_BUF_DATA, SPCM_DIR_CARDTOPC, self.lNotifySize, self.pvBuffer, uint64(0), self.qwBufferSize)


    def run_acquisition(self):
        szErrorTextBuffer = create_string_buffer(ERRORTEXTLEN)
        dwError = uint32()
        trigError = uint32()
        lStatus = int32()
        lAvailUser = int32()
        lPCPos = int32()
        qwTotalMem = uint64(0)
        qwToTransfer = uint64(MEGA_B(8))
        lMaxAdcValue = int32(0)
        trigCounter = int64(0)

        self._set_buffer()

        # start card and DMA
        dwError = spcm_dwSetParam_i32(self.hCard, SPC_M2CMD, M2CMD_CARD_START | M2CMD_CARD_ENABLETRIGGER | M2CMD_DATA_STARTDMA)

        # check for error
        if dwError != 0: # != ERR_OK
            spcm_dwGetErrorInfo_i32(self.hCard, None, None, szErrorTextBuffer)
            sys.stdout.write("{0}\n".format(szErrorTextBuffer.value))
            spcm_vClose(self.hCard)

        # wait until acquisition has finished, then calculated min and max
        else:
            sys.stdout.write("Waiting for trigger...\n")
            trigError = spcm_dwSetParam_i32(self.hCard, SPC_M2CMD, M2CMD_CARD_WAITTRIGGER)

            # Get how many segments have been acquired
            while trigCounter.value < self.n_segments:
                time.sleep(0.1)
                spcm_dwGetParam_i64(self.hCard, SPC_TRIGGERCOUNTER, byref(trigCounter))
                sys.stdout.write("Acquired {0:d}/{1:d}\r".format(trigCounter.value, self.n_segments))

            spcm_dwGetParam_i64(self.hCard, SPC_TRIGGERCOUNTER, byref(trigCounter))
            sys.stdout.write("Acquired {0:d}/{1:d}\n".format(trigCounter.value, self.n_segments))

            dwError = spcm_dwSetParam_i32(self.hCard, SPC_M2CMD, M2CMD_CARD_WAITREADY)
            if dwError != ERR_OK:
                if dwError == ERR_TIMEOUT:
                    sys.stdout.write("... Timeout\n")
                else:
                    sys.stdout.write("... Error: {0:d}\n".format(dwError))

            else:
                lBitsPerSample = int32(0)
                lChCount = int32(0)
                spcm_dwGetParam_i32(self.hCard, SPC_MIINST_BITSPERSAMPLE, byref(lBitsPerSample))
                spcm_dwGetParam_i32(self.hCard, SPC_MIINST_MAXADCVALUE, byref(lMaxAdcValue))

                if lBitsPerSample == 8:
                    pnData = cast(self.pvBuffer, ptr8) # cast to pointer to 8bit integer
                else:
                    pnData = cast(self.pvBuffer, ptr16) # cast to pointer to 16bit integer

                # convert pointer to numpy array
                npData = np.ctypeslib.as_array(pnData, shape=(self.n_segments, self.segment_size, self.n_channels))
                npData = np.transpose(npData, [0, 2, 1])
                vData = npData * np.reshape(self.input_ranges, (1, -1, 1)) / lMaxAdcValue.value
                if self.n_channels == 8:
                    vData = vData[:, [0, 2, 4, 6, 1, 3, 5 ,7]]
                sys.stdout.write ("Finished!\n");
                return vData


    def close(self):
        # clean up
        spcm_vClose(self.hCard)
