import os
import numpy as np
import datetime

from . import m4i, m2i, npz_converter, fcs_data, background_saver


class Manager:

    __instance = None

    def __init__(self, data_dir=None, adc='M2i'):
        self.data_dir = data_dir
        self.bgsaver = background_saver.BackgroundSaver()

        if adc == 'M2i':
            self.adc = m2i.M2i()
        elif adc == 'M4i':
            self.adc = m4i.M4i()
        else:
            print('Specify valid AD board.')


    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance


    def set_channels(self, n_channels=4, channels=None):
        self.channels = channels

        if isinstance(self.adc, m4i.M4i) & (n_channels > 4):
            print('Number of channels must be 4 or lower.')
        if isinstance(self.adc, m2i.M2i) & (n_channels > 8):
            print('Number of channels must be 8 or lower.')

        if channels == None:
            if n_channels <= 1:
                self.channels = (0,)
            elif n_channels <= 2:
                self.channels = (0, 1)
            elif n_channels <= 4:
                self.channels = (0, 1, 2, 3)
            elif n_channels <= 8:
                self.channels = (0, 1, 2, 3, 4, 5, 6, 7)

        self.adc.set_channels(n_channels, channels)
        if isinstance(self.adc, m4i.M4i):
            self.channel_names = ['Ch0', 'Ch1', 'Ch2', 'Ch3']
        elif isinstance(self.adc, m2i.M2i):
            self.channel_names = ['Ch0', 'Ch1', 'Ch2', 'Ch3', 'Ch4', 'Ch5', 'Ch6', 'Ch7']


    def set_input(self, channel, name=None, path='buffered', termination='1M', v_range=1000, coupling='DC', bandwidth='full'):
        if isinstance(self.adc, m2i.M2i):
            self.adc.set_input(channel, termination, v_range)
        elif isinstance(self.adc, m4i.M4i):
            self.adc.set_input(channel, path, termination, v_range, coupling, bandwidth)
        if name is not None:
            self.channel_names[channel] = name


    def set_aquisition(self, segment_size, pretrigger, n_segments, sample_rate=250):
        sample_rate = self.adc.set_aquisition(segment_size, pretrigger, n_segments, sample_rate)
        self.sample_rate = sample_rate


    def set_trigger(self, channel=None, mode='pos', level=0):
        if channel in self.channel_names:
            channel = self.channel_names.index(channel)
        if channel is None:
            print('Specify valid channel.')

        self.adc.set_trigger(channel, mode, level)


    def run_acquisition(self, name=None, on_saved=None):

        if name is None:
            dt = datetime.now()
            name = dt.strftime('%Y%m%d-%H%M%S')

        data = self.adc.run_acquisition()

        if data is None:
            return

        def save():
            saved_place = self._save_data(name, data)
            self.last_saved = saved_place
            print('Saved: {}'.format(saved_place))
            if on_saved is not None:
                on_saved()

        self.bgsaver.add(save)


    def get_last_data(self):
        return self.get_data(self.last_saved)


    def close(self):
        self.adc.close()


    def _get_using_channel_names(self):
        ch_names = []
        for ch in self.channels:
            ch_names.append(self.channel_names[ch])
        return ch_names


    def _save_data(self, file_name, data):
        full_file = os.path.join(self.data_dir, file_name)
        n = 0

        while True:
            fn = full_file + '_{}'.format(n)
            if (os.path.exists(fn + '.npy') or os.path.exists(fn + '.npz')):
                n += 1
            else:
                ch_names = self._get_using_channel_names()
                fcs = fcs_data.FCS_Data.from_data_array(
                                            channel_names=ch_names,
                                            sample_rate=self.sample_rate,
                                            data_array=data)
                fcs.save(fn)
                return fn + '.npz'


    def get_data(self, file_name):
        head, tail = os.path.split(file_name)
        if head is '':
            full_file = os.path.join(self.data_dir, file_name)
        else:
            full_file = file_name

        return fcs_data.FCS_Data.from_file(full_file)
