import numpy as np
import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.path import Path
from matplotlib import patches

from . import gate as gt


def waveform(fcs_data, n=0, t_unit='us', n_channels=None):
    channels = fcs_data.channel_names
    if n_channels is None:
        n_channels = len(channels)
    t_inc = 1 / fcs_data.sample_rate * _get_time_conversion(t_unit)

    f, axes = plt.subplots(n_channels, sharex=True)
    axes[0].set_title('Waveform of n = {}'.format(n))
    leg_list = []
    name_list = []
    for i in range(n_channels):
        ch = channels[i]
        ax = axes[i]
        wf = fcs_data.get_waveform(ch, n)
        t = np.arange(wf.shape[0]) * t_inc
        leg, = ax.plot(t, wf, 'C{}'.format(i))
        leg_list.append(leg)
        name_list.append(ch)

    f.subplots_adjust(hspace=0)
    plt.setp([a.get_xticklabels() for a in f.axes[:-1]], visible=False)

    ax = f.add_subplot(111, frameon=False)
    ax.tick_params(labelcolor='none', top='off', bottom='off', left='off', right='off')
    ax.set_xlabel('Time [{}]'.format(t_unit))
    ax.set_ylabel('Intensity [mV]')
    plt.legend(leg_list, name_list)
    plt.show()


def fsc_ssc(fcs_data, gate=None, n_bins=200, percentile=98, marker=',', show=True):
    x = fcs_data.fsc_a
    y = fcs_data.ssc_a
    density_plot(x, y, gate, n_bins, x_label='FSC-A', y_label='SSC-A', percentile=percentile, marker=marker, show=show)
    return x, y


def fsc(fcs_data, x='H', y='W', gate=None, n_bins=200, percentile=100, marker=',', show=True):
    prefix = 'FSC-'
    return attr_plot(fcs_data, prefix+x, prefix+y, gate, n_bins, percentile=percentile, marker=marker, show=show)


def ssc(fcs_data, x='H', y='W', gate=None, n_bins=200, percentile=100, marker=',', show=True):
    prefix = 'SSC-'
    return attr_plot(fcs_data, prefix+x, prefix+y, gate, n_bins, percentile=percentile, marker=marker, show=show)



def attr_plot(fcs_data, x, y, gate=None, n_bins=200, x_scale='linear', y_scale='linear', percentile=100, marker=',', show=True):
    x_label = x
    y_label = y
    x = fcs_data.get_attr(x)
    y = fcs_data.get_attr(y)
    density_plot(x, y, gate, n_bins, x_label=x_label, y_label=y_label, x_scale=x_scale, y_scale=y_scale, percentile=percentile, marker=marker, show=show)
    return x, y


def density_plot(x, y, gate=None, n_bins=100, x_label='', y_label='', x_scale='linear', y_scale='linear', percentile=100, marker=',', show=True):
    if gate is not None:
        x = x[gate]
        y = y[gate]
    z = _get_hist2d(x, y, n_bins, x_scale, y_scale, percentile)

    # Sort the points by density, so that the densest points are plotted last
    idx = z.argsort()
    X, Y, Z = x[idx], y[idx], z[idx]

    plt.scatter(X, Y, s=1, c=Z, marker=marker, cmap='jet', edgecolors='')
    _set_x_scale(X, x_scale, percentile)
    _set_y_scale(Y, y_scale, percentile)
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    if show:
        plt.show()


def show_gate(axes):
    verts = [[0, 0]]
    patch = patches.PathPatch(Path(verts), edgecolor='red', facecolor='none', lw=2)
    axes.add_patch(patch)
    gate = gt.Gate(patch)
    gate.connect()
    return gate


def _get_hist2d(x, y, n_bins, x_scale, y_scale, percentile):
    x_bins = _get_bins(x, n_bins, x_scale, percentile)
    y_bins = _get_bins(y, n_bins, y_scale, percentile)
    h, x_edge, y_edge = np.histogram2d(x, y, bins=[x_bins, y_bins])
    return np.log(h[_find(x_edge, x), _find(y_edge, y)] + 1)


def _get_bins(x, n_bins, scale, percentile):
    if scale == 'linear':
        return np.linspace(0, np.percentile(x, percentile), n_bins)
    elif scale == 'log':
        return np.logspace(np.log10(np.min(x[x > 0])), np.log10(np.max(x)), n_bins)


def _set_x_scale(x, scale, percentile):
    if scale == 'linear':
        plt.xlim(0, np.percentile(x, percentile))
    elif scale == 'log':
        plt.xscale('log')


def _set_y_scale(y, scale, percentile):
    if scale == 'linear':
        plt.ylim(0, np.percentile(y, percentile))
    elif scale == 'log':
        plt.yscale('log')


def _get_time_conversion(time_unit):
    if time_unit == 'ms':
        return 10 ** 3
    elif time_unit == 'us':
        return 10 ** 6
    elif time_unit == 'ns':
        return 10 ** 9
    else:
        return 1


def _find(array, values):
    idx = np.searchsorted(array, values, side='left')
    l = array.shape[0] - 1
    if type(idx) is np.ndarray:
        idx[idx >= l] = l - 1
    return idx
