import os
import numpy as np
from . import fcs_data

"""
Saves flow cytometry data in compressed npz file.
Contains ndarrays with the following keys.

key: channel_names
String array of channel names.
Used as attributes in FCS_Data class.

key: sampling_interval
Scalar array;
Sampling interval in seconds.

key: time_stamp
1D array [number of segments];
Time stamp of each segment in sampling points.
Multiply by the sampling interval to get the actual time.

key: [each name in channel_names]
2D array [number of segments, segment size];
Voltage data of each channel in millivolts.
"""


def save(fn, array, channel_names=None):
    """
    returns: saved file name
    """

    channels = array.shape[1]

    # check array size and channle_names size
    if channel_names == None:
        np.save(fn, array)
        return fn + '.npy'
    elif not channels == len(channel_names):
        print('Number of channel names does not mach array size.')
        print('Array will be saved in a single npy file.')
        np.save(fn, array)
        return fn + '.npy'

    ch_dict = dict()
    for ch in range(channels):
        ch_dict.update({channel_names[ch]: array[:, ch]})

    np.savez_compressed(fn, **ch_dict)
    return fn + '.npz'


def load(fn):
    root, ext = os.path.splitext(fn)

    if not ext == '':
        return np.load(fn)

    fn_npz = root + '.npz'
    if os.path.exists(fn_npz):
        return np.load(fn_npz)

    fn_npy = root + '.npy'
    if os.path.exists(fn_npy):
        return np.load(fn_npy)

    print('Error: Valid file does not exist.')


def get_combined_data(*files):
    """
    returns: data_keys and data_dict
    """
    loaded = load_files(*files)
    return combine_data(loaded)


def load_files(*files):
    loaded_data = []
    for fn in files:
        if type(fn) is list:
            for f in fn:
                loaded_data.append(load(f))
        else:
            loaded_data.append(load(fn))
    return loaded_data


def combine_data(loaded_data):
    """
    returns: data_keys and data_dict
    """
    data_dict = dict()

    tp = type(loaded_data[0])
    if tp is np.ndarray:
        data = np.concatenate(loaded_data, axis=0)
        data_dict.update({'data': data})
    elif tp is np.lib.npyio.NpzFile:
        for np_dict in loaded_data:
            _add_to_dict(data_dict, np_dict)

    return data_dict


def _add_to_dict(data_dict, npz):
    fcs = fcs_data.FCS_Data
    for key in npz:
        old = data_dict.get(key, None)
        add = npz[key]
        if np.ndim(add) == 0:
            add = add.item()
        if old is None:
            new = add
        else:
            new = _combine_items(key, old, add)

        data_dict.update({key: new})


def _combine_items(key, item1, item2):
    fcs = fcs_data.FCS_Data
    if key in [fcs.KEY_CHANNEL_NAMES, fcs.KEY_SAMPLE_RATE]:
        if np.array_equal(item1, item2) or item1 == item2:
            return item1
        else:
            print('Key:{} does not match.'.format(key))
            raise Exception
    return np.concatenate([item1, item2], axis=0)
