#
# **************************************************************************
#
# simple_rec_single.py                            (c) Spectrum GmbH , 11/2016
#
# Edited by Masashi Ugawa on 2017/09/19
#
# **************************************************************************
#
# Example for all SpcMDrv based (M2i/M3i/M4i) analog acquisition cards.
# Shows a simple Standard mode example using only the few necessary commands
#
# Feel free to use this source for own projects and modify it in any kind
#
# **************************************************************************
#

import sys
import numpy as np
import matplotlib.pyplot as plt

# import spectrum driver functions
from pyspcm import *

#
# **************************************************************************
# szTypeToName: doing name translation
# **************************************************************************
#

def szTypeToName (lCardType):
    sName = ''
    lVersion = (lCardType & TYP_VERSIONMASK)
    if (lCardType & TYP_SERIESMASK) == TYP_M2ISERIES:
        sName = 'M2i.%04x'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M2IEXPSERIES:
        sName = 'M2i.%04x-Exp'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M3ISERIES:
        sName = 'M3i.%04x'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M3IEXPSERIES:
        sName = 'M3i.%04x-Exp'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M4IEXPSERIES:
        sName = 'M4i.%04x-x8'%lVersion
    elif (lCardType & TYP_SERIESMASK) == TYP_M4XEXPSERIES:
        sName = 'M4x.%04x-x4'%lVersion
    else:
        sName = 'unknown type'
    return sName



#
# **************************************************************************
# main
# **************************************************************************
#

szErrorTextBuffer = create_string_buffer (ERRORTEXTLEN)
dwError = uint32 ();
lStatus = int32 ()
lAvailUser = int32 ()
lPCPos = int32 ()
qwTotalMem = uint64 (0);
qwToTransfer = uint64 (MEGA_B(8));
lMaxAdcValue = int32 (0)

# settings for the DMA buffer
lNotifySize = int32 (0); # driver should notify program after all data has been transfered


# open card
hCard = spcm_hOpen (create_string_buffer (b'/dev/spcm0'))
if hCard == None:
    sys.stdout.write("no card found...\n")
    exit ()


# read type, function and sn and check for A/D card
lCardType = int32 (0)
spcm_dwGetParam_i32 (hCard, SPC_PCITYP, byref (lCardType))
lSerialNumber = int32 (0)
spcm_dwGetParam_i32 (hCard, SPC_PCISERIALNO, byref (lSerialNumber))
lFncType = int32 (0)
spcm_dwGetParam_i32 (hCard, SPC_FNCTYPE, byref (lFncType))

sCardName = szTypeToName (lCardType.value)
if lFncType.value == SPCM_TYPE_AI:
    sys.stdout.write("Found: {0} sn {1:05d}\n".format(sCardName,lSerialNumber.value))
else:
    sys.stdout.write("Card: {0} sn {1:05d} not supported by example\n".format(sCardName,lSerialNumber.value))
    exit ()


# do a simple standard setup
# Select channels
n_channels = 4
spcm_dwSetParam_i32(hCard, SPC_CHENABLE, CHANNEL0 | CHANNEL1 | CHANNEL2 | CHANNEL3)

# Set up inputs

# Input paths
# 0: buffered input
# 1: HF 50 ohm input
spcm_dwSetParam_i32(hCard, SPC_PATH0, 0)
spcm_dwSetParam_i32(hCard, SPC_PATH1, 0)
spcm_dwSetParam_i32(hCard, SPC_PATH2, 0)
spcm_dwSetParam_i32(hCard, SPC_PATH3, 0)

# Input ranges
# in millivolts
input_ranges = (200, 200, 200, 200)
spcm_dwSetParam_i32(hCard, SPC_AMP0, input_ranges[0])
spcm_dwSetParam_i32(hCard, SPC_AMP1, input_ranges[1])
spcm_dwSetParam_i32(hCard, SPC_AMP2, input_ranges[2])
spcm_dwSetParam_i32(hCard, SPC_AMP3, input_ranges[3])

# Input coupling
# 0: DC coupling
# 1: AC coupling
spcm_dwSetParam_i32(hCard, SPC_ACDC0, 0)
spcm_dwSetParam_i32(hCard, SPC_ACDC1, 0)
spcm_dwSetParam_i32(hCard, SPC_ACDC2, 0)
spcm_dwSetParam_i32(hCard, SPC_ACDC3, 0)

# Bandwith limit
# 0: full bandwidth
# 1: bandwidth limit (20 MHz)
spcm_dwSetParam_i32(hCard, SPC_FILTER0, 1)
spcm_dwSetParam_i32(hCard, SPC_FILTER1, 1)
spcm_dwSetParam_i32(hCard, SPC_FILTER2, 1)
spcm_dwSetParam_i32(hCard, SPC_FILTER3, 1)

# Aquisition mode
spcm_dwSetParam_i32(hCard, SPC_CARDMODE, SPC_REC_STD_MULTI)
spcm_dwSetParam_i32(hCard, SPC_TIMEOUT, 5000)  # timeout 5 s

# Set sample length
segment_size = 1024
pretrigger = 128
n_segments = 10
mem_size = segment_size * n_segments
spcm_dwSetParam_i32(hCard, SPC_MEMSIZE, mem_size)
spcm_dwSetParam_i32(hCard, SPC_SEGMENTSIZE, segment_size)
spcm_dwSetParam_i32(hCard, SPC_POSTTRIGGER, segment_size - pretrigger)

# Trigger settings
spcm_dwSetParam_i32(hCard, SPC_TRIG_ORMASK, SPC_TMASK_NONE)  # disable software trigger
spcm_dwSetParam_i32(hCard, SPC_TRIG_CH_ORMASK0, SPC_TMASK0_CH0)
spcm_dwSetParam_i32(hCard, SPC_TRIG_CH0_MODE,   SPC_TM_POS)
spcm_dwSetParam_i32(hCard, SPC_TRIG_CH0_LEVEL0, 0)

# Clockmode
spcm_dwSetParam_i32(hCard, SPC_CLOCKMODE,      SPC_CM_INTPLL)          # clock mode internal PLL

# Set the samplerate to division of max (250 MHz) on internal PLL, no clock output
spcm_dwSetParam_i64 (hCard, SPC_SAMPLERATE, MEGA(20))
spcm_dwSetParam_i32 (hCard, SPC_CLOCKOUT, 0)  # no clock output


# define the data buffer
# we try to use continuous memory if available and big enough
qwBufferSize = uint64 (mem_size * 2 * n_channels);
pvBuffer = c_void_p()
qwContBufLen = uint64(0)
spcm_dwGetContBuf_i64(hCard, SPCM_BUF_DATA, byref(pvBuffer), byref(qwContBufLen))
sys.stdout.write("ContBuf length: {0:d}\n".format(qwContBufLen.value))
if qwContBufLen.value >= qwBufferSize.value:
    sys.stdout.write("Using continuous buffer\n")
else:
    pvBuffer = create_string_buffer (qwBufferSize.value)
    sys.stdout.write("Using buffer allocated by user program\n")


spcm_dwDefTransfer_i64 (hCard, SPCM_BUF_DATA, SPCM_DIR_CARDTOPC, lNotifySize, pvBuffer, uint64 (0), qwBufferSize)

# start card and DMA
dwError = spcm_dwSetParam_i32 (hCard, SPC_M2CMD, M2CMD_CARD_START | M2CMD_CARD_ENABLETRIGGER | M2CMD_DATA_STARTDMA)

# check for error
if dwError != 0: # != ERR_OK
    spcm_dwGetErrorInfo_i32 (hCard, None, None, szErrorTextBuffer)
    sys.stdout.write("{0}\n".format(szErrorTextBuffer.value))
    spcm_vClose (hCard)
    exit ()

# wait until acquisition has finished, then calculated min and max
else:
    npData = []
    dwError = spcm_dwSetParam_i32 (hCard, SPC_M2CMD, M2CMD_CARD_WAITREADY)
    if dwError != ERR_OK:
        if dwError == ERR_TIMEOUT:
            sys.stdout.write ("... Timeout\n")
        else:
            sys.stdout.write ("... Error: {0:d}\n".format(dwError))

    else:
        spcm_dwGetParam_i32 (hCard, SPC_M2STATUS,            byref (lStatus))
        spcm_dwGetParam_i32 (hCard, SPC_DATA_AVAIL_USER_LEN, byref (lAvailUser))
        spcm_dwGetParam_i32 (hCard, SPC_DATA_AVAIL_USER_POS, byref (lPCPos))

        qwTotalMem.value = lAvailUser.value
        sys.stdout.write ("Stat:{0:08x} Pos:{1:08x} Avail:{2:08x} Total:{3:.2f}MB/{4:.2f}MB\n".format(lStatus.value, lPCPos.value, lAvailUser.value, c_double (qwTotalMem.value).value / MEGA_B(1), c_double (qwToTransfer.value).value / MEGA_B(1)))

        # this is the point to do anything with the data
        # e.g. calculate minimum and maximum of the acquired data
        lBitsPerSample = int32 (0)
        lChCount = int32 (0)
        spcm_dwGetParam_i32 (hCard, SPC_MIINST_BITSPERSAMPLE, byref (lBitsPerSample))
        spcm_dwGetParam_i32(hCard, SPC_MIINST_MAXADCVALUE, byref (lMaxAdcValue))

        if lBitsPerSample == 8:
            pnData = cast  (pvBuffer, ptr8) # cast to pointer to 8bit integer
        else:
            pnData = cast  (pvBuffer, ptr16) # cast to pointer to 16bit integer

        for i in range (0, mem_size * n_channels, 1):
            npData.append(pnData[i])

        sys.stdout.write ("Finished...\n");

# clean up
spcm_vClose (hCard)

npData = np.asarray(npData)
npData = np.transpose(np.reshape(npData, (-1, n_channels)))

voltageData = npData * np.reshape(np.asarray(input_ranges), (-1, 1)) / lMaxAdcValue.value
splitData = np.asarray(np.split(voltageData, n_segments, axis=1))

print(splitData.shape)

n = 0
plt.plot(splitData[n, 0])
plt.plot(splitData[n, 1])
plt.plot(splitData[n, 2])
plt.plot(splitData[n, 3])
plt.show()
