import sys
import unittest
import numpy as np

from spectrum_api.fcs_data import FCS_Data


class TestFcsData(unittest.TestCase):


    def test_constructor(self):
        channel_names = ['ch0', 'ch1']
        sample_rate = 1000
        ch0 = np.random.rand(100, 1000)
        ch1 = np.random.rand(100, 1000)
        data_dict = {channel_names[0]: ch0,
                     channel_names[1]: ch1}

        fcs = FCS_Data(channel_names, sample_rate, **data_dict)

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        np.testing.assert_array_equal(fcs.ch0, ch0)
        np.testing.assert_array_equal(fcs.ch1, ch1)


    def test_array_constructor(self):
        channel_names = ['ch0', 'ch1']
        sample_rate = 1000
        data = np.random.rand(100, 2, 1000)
        ch0 = data[:, 0]
        ch1 = data[:, 1]

        fcs = FCS_Data.from_data_array(channel_names, sample_rate, data)

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        self.assertTupleEqual(fcs.ch0.shape, ch0.shape)
        np.testing.assert_array_equal(fcs.ch0, ch0)
        np.testing.assert_array_equal(fcs.ch1, ch1)


    def test_file_constructor(self):
        fcs = self._create_fcs()
        fn = 'test'
        fcs.save(fn)
        fcs_actual = FCS_Data.from_file(fn)
        self._assert_fcs_equal(fcs, fcs_actual)


    def _assert_fcs_equal(self, a, b):
        self.assertListEqual(a.channel_names, b.channel_names)
        self.assertEqual(a.sample_rate, b.sample_rate)
        for key in a.channel_names:
            np.testing.assert_array_equal(getattr(a, key), getattr(b, key))


    def _create_fcs(self):
        channel_names = ['ch0', 'ch1']
        sample_rate = 1000
        data = np.random.rand(100, 2, 1000)
        ch0 = data[:, 0]
        ch1 = data[:, 1]
        return FCS_Data.from_data_array(channel_names, sample_rate, data)


    def test_constructor_ssc_fsc(self):
        channel_names = ['FSC', 'SSC']
        sample_rate = 1000
        ch0 = np.random.rand(100, 1000)
        ch1 = np.random.rand(100, 1000)
        data_dict = {channel_names[0]: ch0,
                     channel_names[1]: ch1}

        fcs = FCS_Data(channel_names, sample_rate, **data_dict)

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        np.testing.assert_array_equal(fcs.fsc, ch0)
        np.testing.assert_array_equal(fcs.ssc, ch1)


    def test_constructor_calcein_spc_mod(self):
        channel_names = ['Calcein AM', 'SPC-mod']
        sample_rate = 1000
        ch0 = np.random.rand(100, 1000)
        ch1 = np.random.rand(100, 1000)
        data_dict = {channel_names[0]: ch0,
                     channel_names[1]: ch1}

        fcs = FCS_Data(channel_names, sample_rate, **data_dict)

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        np.testing.assert_array_equal(fcs.calcein_am, ch0)
        np.testing.assert_array_equal(fcs.spc_mod, ch1)

    def test_array_constructor_FSC_SSC(self):
        channel_names = ['FSC', 'SSC']
        sample_rate = 1000
        data = np.random.rand(100, 2, 1000)
        ch0 = data[:, 0]
        ch1 = data[:, 1]

        fcs = FCS_Data.from_data_array(channel_names, sample_rate, data)

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        self.assertTupleEqual(fcs.fsc.shape, ch0.shape)
        np.testing.assert_array_equal(fcs.fsc, ch0)
        np.testing.assert_array_equal(fcs.ssc, ch1)


    def test_constructor_ssc_fsc(self):
        channel_names = ['FSC', 'SSC']
        sample_rate = 1000
        ch0 = np.random.rand(100, 1000)
        ch1 = np.random.rand(100, 1000)
        data_dict = {channel_names[0]: ch0,
                     channel_names[1]: ch1}

        fcs = FCS_Data(channel_names, sample_rate, **data_dict)
        fcs.create_subattributes()

        self.assertListEqual(fcs.channel_names, channel_names)
        self.assertEqual(fcs.sample_rate, sample_rate)
        np.testing.assert_array_equal(fcs.fsc, ch0)
        np.testing.assert_array_equal(fcs.ssc, ch1)
        self.assertTrue(fcs.fsc_a is not None)


if __name__ == '__main__':
    unittest.main()
