import sys
import unittest
import numpy as np

from spectrum_api import npz_converter
from spectrum_api.fcs_data import FCS_Data


class TestNpzConverter(unittest.TestCase):


    def test_load(self):
        fn = 'test.npz'
        loaded = npz_converter.load(fn)
        self.assertTrue(type(loaded) is np.lib.npyio.NpzFile)

        fn = 'test'
        loaded = npz_converter.load(fn)
        self.assertTrue(type(loaded) is np.lib.npyio.NpzFile)


    def test_load_files_single(self):
        fn = 'test'
        loaded_files = npz_converter.load_files(fn)
        self.assertTrue(type(loaded_files) is list)
        self.assertEqual(len(loaded_files), 1)
        for loaded in loaded_files:
            self.assertTrue(type(loaded) is np.lib.npyio.NpzFile)

    def test_load_files_multiple_args(self):
        fn = 'test'
        loaded_files = npz_converter.load_files(fn, fn)
        self.assertTrue(type(loaded_files) is list)
        self.assertEqual(len(loaded_files), 2)
        for loaded in loaded_files:
            self.assertTrue(type(loaded) is np.lib.npyio.NpzFile)

    def test_load_files_list_arg(self):
        fn = 'test'
        loaded_files = npz_converter.load_files([fn, fn, fn])
        self.assertTrue(type(loaded_files) is list)
        self.assertEqual(len(loaded_files), 3)
        for loaded in loaded_files:
            self.assertTrue(type(loaded) is np.lib.npyio.NpzFile)


    def test_combine_arrays(self):
        key = 'ch0'
        item1 = np.random.rand(10, 100)
        item2 = np.random.rand(10, 100)

        combined = npz_converter._combine_items(key, item1, item2)
        np.testing.assert_array_equal(combined, np.concatenate([item1, item2], axis=0))


    def test_combine_channel_names(self):
        key = FCS_Data.KEY_CHANNEL_NAMES
        item1 = ['ch0', 'ch1']
        item2 = ['ch0', 'ch1']

        combined = npz_converter._combine_items(key, item1, item2)
        self.assertListEqual(combined, item1)


    def test_combine_sample_rate(self):
        key = FCS_Data.KEY_SAMPLE_RATE
        item1 = 1000
        item2 = 1000

        combined = npz_converter._combine_items(key, item1, item2)
        self.assertEqual(combined, item1)

    def test_add_to_dict(self):
        key1 = FCS_Data.KEY_CHANNEL_NAMES
        key2 = FCS_Data.KEY_SAMPLE_RATE
        channel_names = ['ch0', 'ch1']
        sample_rate = 1000
        item1 = np.asarray(channel_names)
        item2 = np.asarray(sample_rate)
        dict1 = {key1: item1, key2: item2}
        dict2 = {key1: item1, key2: item2}

        npz_converter._add_to_dict(dict1, dict2)
        self.assertEqual(dict1.keys(), dict2.keys())
        np.testing.assert_array_equal(dict1[key1], channel_names)
        np.testing.assert_array_equal(dict1[key2], sample_rate)


    def test_add_to_dict_array(self):
        key = 'ch0'
        array1 = np.random.rand(10, 100)
        array2 = np.random.rand(10, 100)
        dict1 = {key: array1}
        dict2 = {key: array2}

        npz_converter._add_to_dict(dict1, dict2)
        self.assertEqual(dict1.keys(), dict2.keys())
        np.testing.assert_array_equal(dict1[key], np.concatenate([array1, array2], axis=0))


    def test_combine_data_single(self):
        fn = 'test'
        loaded_files = npz_converter.load_files(fn)
        data_dict = npz_converter.combine_data(loaded_files)

        self.assertTrue(type(data_dict) is dict)
        self.assertTrue(FCS_Data.KEY_SAMPLE_RATE in data_dict.keys())
        self.assertTrue(FCS_Data.KEY_CHANNEL_NAMES in data_dict.keys())

        for key in data_dict.get(FCS_Data.KEY_CHANNEL_NAMES):
            self.assertTrue(key in data_dict.keys())

    def test_combine_data_multiple(self):
        fn = 'test'
        loaded_files = npz_converter.load_files(fn, fn)
        data_dict = npz_converter.combine_data(loaded_files)

        self.assertTrue(type(data_dict) is dict)
        self.assertTrue(FCS_Data.KEY_SAMPLE_RATE in data_dict.keys())
        self.assertTrue(FCS_Data.KEY_CHANNEL_NAMES in data_dict.keys())
        for key in data_dict.get(FCS_Data.KEY_CHANNEL_NAMES):
            self.assertTrue(key in data_dict.keys())


if __name__ == '__main__':
    unittest.main()
